<?php
namespace WP_Media_Folder\Aws\Ec2;

use WP_Media_Folder\Aws\AwsClient;
use WP_Media_Folder\Aws\Api\Service;
use WP_Media_Folder\Aws\Api\DocModel;
use WP_Media_Folder\Aws\Api\ApiProvider;
use WP_Media_Folder\Aws\PresignUrlMiddleware;

/**
 * Client used to interact with Amazon EC2.
 *
 * @method \WP_Media_Folder\Aws\Result acceptVpcPeeringConnection(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise acceptVpcPeeringConnectionAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result allocateAddress(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise allocateAddressAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result allocateHosts(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise allocateHostsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result assignPrivateIpAddresses(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise assignPrivateIpAddressesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result associateAddress(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise associateAddressAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result associateDhcpOptions(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise associateDhcpOptionsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result associateRouteTable(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise associateRouteTableAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result attachClassicLinkVpc(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise attachClassicLinkVpcAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result attachInternetGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise attachInternetGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result attachNetworkInterface(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise attachNetworkInterfaceAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result attachVolume(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise attachVolumeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result attachVpnGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise attachVpnGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result authorizeSecurityGroupEgress(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise authorizeSecurityGroupEgressAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result authorizeSecurityGroupIngress(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise authorizeSecurityGroupIngressAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result bundleInstance(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise bundleInstanceAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result cancelBundleTask(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise cancelBundleTaskAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result cancelConversionTask(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise cancelConversionTaskAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result cancelExportTask(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise cancelExportTaskAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result cancelImportTask(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise cancelImportTaskAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result cancelReservedInstancesListing(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise cancelReservedInstancesListingAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result cancelSpotFleetRequests(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise cancelSpotFleetRequestsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result cancelSpotInstanceRequests(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise cancelSpotInstanceRequestsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result confirmProductInstance(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise confirmProductInstanceAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result copyImage(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise copyImageAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result copySnapshot(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise copySnapshotAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createCustomerGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createCustomerGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createDhcpOptions(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createDhcpOptionsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createFlowLogs(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createFlowLogsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createImage(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createImageAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createInstanceExportTask(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createInstanceExportTaskAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createInternetGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createInternetGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createKeyPair(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createKeyPairAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createNatGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createNatGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createNetworkAcl(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createNetworkAclAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createNetworkAclEntry(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createNetworkAclEntryAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createNetworkInterface(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createNetworkInterfaceAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createPlacementGroup(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createPlacementGroupAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createReservedInstancesListing(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createReservedInstancesListingAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createRoute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createRouteAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createRouteTable(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createRouteTableAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createSecurityGroup(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createSecurityGroupAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createSnapshot(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createSnapshotAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createSpotDatafeedSubscription(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createSpotDatafeedSubscriptionAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createSubnet(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createSubnetAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createTags(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createTagsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createVolume(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createVolumeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createVpc(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createVpcAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createVpcEndpoint(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createVpcEndpointAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createVpcPeeringConnection(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createVpcPeeringConnectionAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createVpnConnection(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createVpnConnectionAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createVpnConnectionRoute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createVpnConnectionRouteAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result createVpnGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createVpnGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteCustomerGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteCustomerGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteDhcpOptions(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteDhcpOptionsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteFlowLogs(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteFlowLogsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteInternetGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteInternetGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteKeyPair(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteKeyPairAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteNatGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteNatGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteNetworkAcl(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteNetworkAclAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteNetworkAclEntry(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteNetworkAclEntryAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteNetworkInterface(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteNetworkInterfaceAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deletePlacementGroup(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deletePlacementGroupAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteRoute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteRouteAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteRouteTable(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteRouteTableAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteSecurityGroup(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteSecurityGroupAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteSnapshot(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteSnapshotAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteSpotDatafeedSubscription(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteSpotDatafeedSubscriptionAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteSubnet(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteSubnetAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteTags(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteTagsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteVolume(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteVolumeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteVpc(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteVpcAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteVpcEndpoints(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteVpcEndpointsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteVpcPeeringConnection(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteVpcPeeringConnectionAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteVpnConnection(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteVpnConnectionAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteVpnConnectionRoute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteVpnConnectionRouteAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deleteVpnGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteVpnGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result deregisterImage(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deregisterImageAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeAccountAttributes(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeAccountAttributesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeAddresses(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeAddressesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeAvailabilityZones(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeAvailabilityZonesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeBundleTasks(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeBundleTasksAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeClassicLinkInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeClassicLinkInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeConversionTasks(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeConversionTasksAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeCustomerGateways(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeCustomerGatewaysAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeDhcpOptions(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeDhcpOptionsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeExportTasks(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeExportTasksAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeFlowLogs(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeFlowLogsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeHosts(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeHostsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeIdFormat(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeIdFormatAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeImageAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeImageAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeImages(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeImagesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeImportImageTasks(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeImportImageTasksAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeImportSnapshotTasks(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeImportSnapshotTasksAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeInstanceAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeInstanceAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeInstanceStatus(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeInstanceStatusAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeInternetGateways(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeInternetGatewaysAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeKeyPairs(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeKeyPairsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeMovingAddresses(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeMovingAddressesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeNatGateways(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeNatGatewaysAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeNetworkAcls(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeNetworkAclsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeNetworkInterfaceAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeNetworkInterfaceAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeNetworkInterfaces(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeNetworkInterfacesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describePlacementGroups(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describePlacementGroupsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describePrefixLists(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describePrefixListsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeRegions(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeRegionsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeReservedInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeReservedInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeReservedInstancesListings(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeReservedInstancesListingsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeReservedInstancesModifications(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeReservedInstancesModificationsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeReservedInstancesOfferings(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeReservedInstancesOfferingsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeRouteTables(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeRouteTablesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeScheduledInstanceAvailability(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeScheduledInstanceAvailabilityAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeScheduledInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeScheduledInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSecurityGroupReferences(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSecurityGroupReferencesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSecurityGroups(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSecurityGroupsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSnapshotAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSnapshotAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSnapshots(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSnapshotsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSpotDatafeedSubscription(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSpotDatafeedSubscriptionAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSpotFleetInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSpotFleetInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSpotFleetRequestHistory(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSpotFleetRequestHistoryAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSpotFleetRequests(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSpotFleetRequestsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSpotInstanceRequests(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSpotInstanceRequestsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSpotPriceHistory(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSpotPriceHistoryAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeStaleSecurityGroups(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeStaleSecurityGroupsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeSubnets(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeSubnetsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeTags(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeTagsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVolumeAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVolumeAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVolumeStatus(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVolumeStatusAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVolumes(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVolumesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVpcAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVpcClassicLink(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcClassicLinkAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVpcClassicLinkDnsSupport(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcClassicLinkDnsSupportAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVpcEndpointServices(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcEndpointServicesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVpcEndpoints(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcEndpointsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVpcPeeringConnections(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcPeeringConnectionsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVpcs(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVpnConnections(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpnConnectionsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeVpnGateways(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpnGatewaysAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result detachClassicLinkVpc(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise detachClassicLinkVpcAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result detachInternetGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise detachInternetGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result detachNetworkInterface(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise detachNetworkInterfaceAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result detachVolume(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise detachVolumeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result detachVpnGateway(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise detachVpnGatewayAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result disableVgwRoutePropagation(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disableVgwRoutePropagationAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result disableVpcClassicLink(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disableVpcClassicLinkAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result disableVpcClassicLinkDnsSupport(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disableVpcClassicLinkDnsSupportAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result disassociateAddress(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disassociateAddressAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result disassociateRouteTable(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disassociateRouteTableAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result enableVgwRoutePropagation(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise enableVgwRoutePropagationAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result enableVolumeIO(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise enableVolumeIOAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result enableVpcClassicLink(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise enableVpcClassicLinkAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result enableVpcClassicLinkDnsSupport(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise enableVpcClassicLinkDnsSupportAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result getConsoleOutput(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise getConsoleOutputAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result getConsoleScreenshot(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise getConsoleScreenshotAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result getPasswordData(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise getPasswordDataAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result importImage(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise importImageAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result importInstance(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise importInstanceAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result importKeyPair(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise importKeyPairAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result importSnapshot(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise importSnapshotAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result importVolume(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise importVolumeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyHosts(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyHostsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyIdFormat(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyIdFormatAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyImageAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyImageAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyInstanceAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyInstanceAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyInstancePlacement(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyInstancePlacementAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyNetworkInterfaceAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyNetworkInterfaceAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyReservedInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyReservedInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifySnapshotAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifySnapshotAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifySpotFleetRequest(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifySpotFleetRequestAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifySubnetAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifySubnetAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyVolumeAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyVolumeAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyVpcAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyVpcAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyVpcEndpoint(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyVpcEndpointAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result modifyVpcPeeringConnectionOptions(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyVpcPeeringConnectionOptionsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result monitorInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise monitorInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result moveAddressToVpc(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise moveAddressToVpcAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result purchaseReservedInstancesOffering(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise purchaseReservedInstancesOfferingAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result purchaseScheduledInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise purchaseScheduledInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result rebootInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise rebootInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result registerImage(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise registerImageAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result rejectVpcPeeringConnection(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise rejectVpcPeeringConnectionAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result releaseAddress(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise releaseAddressAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result releaseHosts(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise releaseHostsAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result replaceNetworkAclAssociation(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise replaceNetworkAclAssociationAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result replaceNetworkAclEntry(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise replaceNetworkAclEntryAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result replaceRoute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise replaceRouteAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result replaceRouteTableAssociation(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise replaceRouteTableAssociationAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result reportInstanceStatus(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise reportInstanceStatusAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result requestSpotFleet(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise requestSpotFleetAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result requestSpotInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise requestSpotInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result resetImageAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise resetImageAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result resetInstanceAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise resetInstanceAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result resetNetworkInterfaceAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise resetNetworkInterfaceAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result resetSnapshotAttribute(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise resetSnapshotAttributeAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result restoreAddressToClassic(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise restoreAddressToClassicAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result revokeSecurityGroupEgress(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise revokeSecurityGroupEgressAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result revokeSecurityGroupIngress(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise revokeSecurityGroupIngressAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result runInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise runInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result runScheduledInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise runScheduledInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result startInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise startInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result stopInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise stopInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result terminateInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise terminateInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result unassignPrivateIpAddresses(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise unassignPrivateIpAddressesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result unmonitorInstances(array $args = [])
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise unmonitorInstancesAsync(array $args = [])
 * @method \WP_Media_Folder\Aws\Result describeHostReservationOfferings(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeHostReservationOfferingsAsync(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeHostReservations(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeHostReservationsAsync(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeIdentityIdFormat(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeIdentityIdFormatAsync(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result getHostReservationPurchasePreview(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise getHostReservationPurchasePreviewAsync(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyIdentityIdFormat(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyIdentityIdFormatAsync(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result purchaseHostReservation(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise purchaseHostReservationAsync(array $args = []) (supported in versions 2016-04-01, 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result acceptReservedInstancesExchangeQuote(array $args = []) (supported in versions 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise acceptReservedInstancesExchangeQuoteAsync(array $args = []) (supported in versions 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result getReservedInstancesExchangeQuote(array $args = []) (supported in versions 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise getReservedInstancesExchangeQuoteAsync(array $args = []) (supported in versions 2016-09-15, 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result acceptTransitGatewayVpcAttachment(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise acceptTransitGatewayVpcAttachmentAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result acceptVpcEndpointConnections(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise acceptVpcEndpointConnectionsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result advertiseByoipCidr(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise advertiseByoipCidrAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result assignIpv6Addresses(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise assignIpv6AddressesAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result associateIamInstanceProfile(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise associateIamInstanceProfileAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result associateSubnetCidrBlock(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise associateSubnetCidrBlockAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result associateTransitGatewayRouteTable(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise associateTransitGatewayRouteTableAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result associateVpcCidrBlock(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise associateVpcCidrBlockAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result cancelCapacityReservation(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise cancelCapacityReservationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result copyFpgaImage(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise copyFpgaImageAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createCapacityReservation(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createCapacityReservationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createDefaultSubnet(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createDefaultSubnetAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createDefaultVpc(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createDefaultVpcAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createEgressOnlyInternetGateway(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createEgressOnlyInternetGatewayAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createFleet(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createFleetAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createFpgaImage(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createFpgaImageAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createLaunchTemplate(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createLaunchTemplateAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createLaunchTemplateVersion(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createLaunchTemplateVersionAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createNetworkInterfacePermission(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createNetworkInterfacePermissionAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createTransitGateway(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createTransitGatewayAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createTransitGatewayRoute(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createTransitGatewayRouteAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createTransitGatewayRouteTable(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createTransitGatewayRouteTableAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createTransitGatewayVpcAttachment(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createTransitGatewayVpcAttachmentAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createVpcEndpointConnectionNotification(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createVpcEndpointConnectionNotificationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result createVpcEndpointServiceConfiguration(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise createVpcEndpointServiceConfigurationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteEgressOnlyInternetGateway(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteEgressOnlyInternetGatewayAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteFleets(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteFleetsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteFpgaImage(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteFpgaImageAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteLaunchTemplate(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteLaunchTemplateAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteLaunchTemplateVersions(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteLaunchTemplateVersionsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteNetworkInterfacePermission(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteNetworkInterfacePermissionAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteTransitGateway(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteTransitGatewayAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteTransitGatewayRoute(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteTransitGatewayRouteAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteTransitGatewayRouteTable(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteTransitGatewayRouteTableAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteTransitGatewayVpcAttachment(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteTransitGatewayVpcAttachmentAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteVpcEndpointConnectionNotifications(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteVpcEndpointConnectionNotificationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deleteVpcEndpointServiceConfigurations(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deleteVpcEndpointServiceConfigurationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result deprovisionByoipCidr(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise deprovisionByoipCidrAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeAggregateIdFormat(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeAggregateIdFormatAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeByoipCidrs(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeByoipCidrsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeCapacityReservations(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeCapacityReservationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeEgressOnlyInternetGateways(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeEgressOnlyInternetGatewaysAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeElasticGpus(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeElasticGpusAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeFleetHistory(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeFleetHistoryAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeFleetInstances(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeFleetInstancesAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeFleets(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeFleetsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeFpgaImageAttribute(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeFpgaImageAttributeAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeFpgaImages(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeFpgaImagesAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeIamInstanceProfileAssociations(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeIamInstanceProfileAssociationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeInstanceCreditSpecifications(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeInstanceCreditSpecificationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeLaunchTemplateVersions(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeLaunchTemplateVersionsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeLaunchTemplates(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeLaunchTemplatesAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeNetworkInterfacePermissions(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeNetworkInterfacePermissionsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describePrincipalIdFormat(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describePrincipalIdFormatAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describePublicIpv4Pools(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describePublicIpv4PoolsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeTransitGatewayAttachments(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeTransitGatewayAttachmentsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeTransitGatewayRouteTables(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeTransitGatewayRouteTablesAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeTransitGatewayVpcAttachments(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeTransitGatewayVpcAttachmentsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeTransitGateways(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeTransitGatewaysAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeVolumesModifications(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVolumesModificationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeVpcEndpointConnectionNotifications(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcEndpointConnectionNotificationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeVpcEndpointConnections(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcEndpointConnectionsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeVpcEndpointServiceConfigurations(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcEndpointServiceConfigurationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result describeVpcEndpointServicePermissions(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise describeVpcEndpointServicePermissionsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result disableTransitGatewayRouteTablePropagation(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disableTransitGatewayRouteTablePropagationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result disassociateIamInstanceProfile(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disassociateIamInstanceProfileAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result disassociateSubnetCidrBlock(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disassociateSubnetCidrBlockAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result disassociateTransitGatewayRouteTable(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disassociateTransitGatewayRouteTableAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result disassociateVpcCidrBlock(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise disassociateVpcCidrBlockAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result enableTransitGatewayRouteTablePropagation(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise enableTransitGatewayRouteTablePropagationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result exportTransitGatewayRoutes(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise exportTransitGatewayRoutesAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result getLaunchTemplateData(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise getLaunchTemplateDataAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result getTransitGatewayAttachmentPropagations(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise getTransitGatewayAttachmentPropagationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result getTransitGatewayRouteTableAssociations(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise getTransitGatewayRouteTableAssociationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result getTransitGatewayRouteTablePropagations(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise getTransitGatewayRouteTablePropagationsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyCapacityReservation(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyCapacityReservationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyFleet(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyFleetAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyFpgaImageAttribute(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyFpgaImageAttributeAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyInstanceCapacityReservationAttributes(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyInstanceCapacityReservationAttributesAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyInstanceCreditSpecification(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyInstanceCreditSpecificationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyLaunchTemplate(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyLaunchTemplateAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyTransitGatewayVpcAttachment(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyTransitGatewayVpcAttachmentAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyVolume(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyVolumeAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyVpcEndpointConnectionNotification(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyVpcEndpointConnectionNotificationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyVpcEndpointServiceConfiguration(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyVpcEndpointServiceConfigurationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyVpcEndpointServicePermissions(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyVpcEndpointServicePermissionsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result modifyVpcTenancy(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise modifyVpcTenancyAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result provisionByoipCidr(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise provisionByoipCidrAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result rejectTransitGatewayVpcAttachment(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise rejectTransitGatewayVpcAttachmentAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result rejectVpcEndpointConnections(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise rejectVpcEndpointConnectionsAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result replaceIamInstanceProfileAssociation(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise replaceIamInstanceProfileAssociationAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result replaceTransitGatewayRoute(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise replaceTransitGatewayRouteAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result resetFpgaImageAttribute(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise resetFpgaImageAttributeAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result searchTransitGatewayRoutes(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise searchTransitGatewayRoutesAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result unassignIpv6Addresses(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise unassignIpv6AddressesAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result updateSecurityGroupRuleDescriptionsEgress(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise updateSecurityGroupRuleDescriptionsEgressAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result updateSecurityGroupRuleDescriptionsIngress(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise updateSecurityGroupRuleDescriptionsIngressAsync(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\Aws\Result withdrawByoipCidr(array $args = []) (supported in versions 2016-11-15)
 * @method \WP_Media_Folder\GuzzleHttp\Promise\Promise withdrawByoipCidrAsync(array $args = []) (supported in versions 2016-11-15)
 */
class Ec2Client extends AwsClient
{
    public function __construct(array $args)
    {
        $args['with_resolved'] = function (array $args) {
            $this->getHandlerList()->appendInit(
                PresignUrlMiddleware::wrap(
                    $this,
                    $args['endpoint_provider'],
                    [
                        'operations' => [
                            'CopySnapshot',
                        ],
                        'service' => 'ec2',
                        'presign_param' => 'PresignedUrl',
                    ]
                ),
                'ec2.copy_snapshot'
            );
        };

        parent::__construct($args);
    }

    /**
     * @internal
     * @codeCoverageIgnore
     */
    public static function applyDocFilters(array $api, array $docs)
    {
        // Several copy snapshot parameters are optional.
        $docs['shapes']['String']['refs']['CopySnapshotRequest$PresignedUrl']
            = '<div class="alert alert-info">The SDK will compute this value '
            . 'for you on your behalf.</div>';
        $docs['shapes']['String']['refs']['CopySnapshotRequest$DestinationRegion']
            = '<div class="alert alert-info">The SDK will populate this '
            . 'parameter on your behalf using the configured region value of '
            . 'the client.</div>';

        return [
            new Service($api, ApiProvider::defaultProvider()),
            new DocModel($docs)
        ];
    }
}
